# dynamic_test_runner.py
import os
import tempfile
from typing import IO, Awaitable, Callable, List

import pytest
from pydantic import BaseModel

from bananalyzer.data.schemas import Example

TestType = Callable[[], Awaitable[None]]


class BananalyzerTest(BaseModel):
    code: str
    example: Example


def generate_test(example: Example, headless: bool) -> BananalyzerTest:
    return BananalyzerTest(
        code=f"""
@pytest.mark.asyncio
async def test_{example.id.replace("-", "_")}() -> None:
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless={headless})
        context = await browser.new_context()
        example = get_example_by_url("{example.url}")
         
        # The agent is imported into the global context prior to this call 
        result = await agent.run(context, example)
        for curr_eval in example.evals:
            curr_eval.eval_results(result)
""",
        example=example,
    )


def write_load_agent_module(f: IO[str], file_path: str) -> None:
    f.write(
        f"""
import importlib.util
import sys
from pathlib import Path

def load_agent_module():
    path = Path("{file_path}")
    module_name = path.stem

    spec = importlib.util.spec_from_file_location(module_name, "{file_path}")
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    globals()["agent"] = getattr(module, "agent")
load_agent_module()
"""
    )


def create_test_file(
    tests: List[BananalyzerTest], prefix: str, agent_file_path: str
) -> str:
    with tempfile.NamedTemporaryFile(
        mode="w+", delete=False, prefix=prefix, suffix=".py"
    ) as f:
        f.write("import pytest\n")
        f.write("from bananalyzer.data.examples import get_example_by_url\n")
        f.write("from playwright.async_api import async_playwright\n\n")
        f.write("global test\n")
        f.write("test = 'test'\n")
        write_load_agent_module(f, agent_file_path)

        for test_content in tests:
            f.write(f"{test_content.code}\n\n")

    return f.name


def run_tests(
    tests: List[BananalyzerTest], agent_file_path: str, no_capture: bool
) -> int:
    """
    Create temporary test files based on intent, run them, and then delete them
    """
    intents = {test.example.type for test in tests}
    intent_separated_tests = [
        [test for test in tests if test.example.type == intent] for intent in intents
    ]
    test_file_names = [
        create_test_file(tests, f"{tests[0].example.type}_intent_", agent_file_path)
        for tests in intent_separated_tests
    ]

    try:
        return_code = pytest.main(test_file_names + (["-s"] if no_capture else []))
    finally:
        for test_file_name in test_file_names:
            os.unlink(test_file_name)

    return return_code



from typing import Optional

from pydantic import BaseModel, Field

from bananalyzer import GoalType


class Args(BaseModel):
    path: str
    headless: bool
    id: Optional[str] = Field(default=None)
    intent: Optional[GoalType] = Field(default=None)
    domain: Optional[str] = Field(default=None)
    s: bool



import asyncio
import os
from typing import Any, List

from playwright.async_api import async_playwright

from bananalyzer import examples
from bananalyzer.data.schemas import Example

"""
Test script to re-load websites and save them as MHTML files.
"""


async def save_page_as_mhtml(client: Any, page: Any, file_path: str) -> None:
    # Wait for 2 seconds
    await asyncio.sleep(3)

    # Check if the file exists before trying to delete it
    if os.path.exists(file_path):
        os.remove(file_path)

    # Save the page as MHTML
    result = await client.send("Page.captureSnapshot", {"format": "mhtml"})
    mhtml = result["data"]
    with open(file_path, "w") as file:
        file.write(mhtml)


async def setup_and_save(examples: List[Example]) -> None:
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=False)
        context = await browser.new_context()
        page = await context.new_page()
        client = await context.new_cdp_session(page)

        for item in examples:
            print(item.id, item.url)
            folder_path: str = f"./python/bananalyzer/data/{item.id}"
            os.makedirs(folder_path, exist_ok=True)
            file_path: str = os.path.join(folder_path, "index.mhtml")

            await page.goto(item.url)
            await save_page_as_mhtml(client, page, file_path)

        await browser.close()


if __name__ == "__main__":
    print(examples)
    asyncio.run(setup_and_save(examples))



# Separate banana-lyzer args from pytest args
# Look for an instance of Banana-lyzer in the current directory
# If it doesn't exist, error
import argparse
import importlib.util
import os
import sys
from pathlib import Path
from typing import Any

from bananalyzer.data.examples import examples
from bananalyzer.data.schemas import GoalType
from bananalyzer.runner.agent_runner import AgentRunner
from bananalyzer.runner.runner import generate_test, run_tests
from bananalyzer.schema import Args


def print_intro() -> None:
    # https://www.asciiart.eu/food-and-drinks/bananas
    print(
        """//\ 
V  \ 
 \  \_
  \,'.`-.
   |\ `. `.       
   ( \  `. `-.                        _,.-:\ 
    \ \   `.  `-._             __..--' ,-';/
     \ `.   `-.   `-..___..---'   _.--' ,'/
      `. `.    `-._        __..--'    ,' /
        `. `-_     ``--..''       _.-' ,'
          `-_ `-.___        __,--'   ,'
             `-.__  `----'''    __.-'
                  `--..____..--'
"""
    )
    print("Bananalyzing... 🍌")


def parse_args() -> Args:
    file_name = "bananalyzer-agent.py"
    parser = argparse.ArgumentParser(
        description=f"Run the agent inside a bananalyzer agent definition file against the benchmark",
    )
    parser.add_argument("path", type=str, help=f"Path to the {file_name} file")
    parser.add_argument(
        "--headless", action="store_true", help=f"Whether to run headless or not"
    )
    parser.add_argument(
        "-s",
        "--s",
        action="store_true",
        help="Shortcut for --capture=no in pytest. Will print stdout and stderr",
    )
    parser.add_argument(
        "-id",
        "--id",
        type=str,
        default=None,
        help="Filter tests by id",
    )
    parser.add_argument(
        "-i",
        "--intent",
        type=str,
        default=None,
        help="Filter tests by a particular intent",
    )
    parser.add_argument(
        "-d",
        "--domain",
        type=str,
        default=None,
        help="Filter tests by a particular domain",
    )

    args = parser.parse_args()

    file_name = os.path.basename(args.path)
    if file_name != file_name:
        raise RuntimeError(f"The provided file name must be {file_name}")

    return Args(
        path=args.path,
        headless=args.headless,
        intent=args.intent,
        id=args.id,
        domain=args.domain,
        s=args.s,
    )


def load_agent_from_path(file_path: str) -> Any:
    path = Path(file_path)
    module_name = path.stem

    spec = importlib.util.spec_from_file_location(module_name, file_path)
    if spec is None or spec.loader is None:
        raise ImportError(f"Cannot load module from path {file_path}")
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)

    return getattr(module, "agent")


def validate_agent_instance_available(agent: Any) -> None:
    # Ensures there is some "agent" variable in the global context
    # This is a bit hacky, but it's the best way to ensure the agent is available
    # without actually running the file
    if not isinstance(agent, AgentRunner):
        raise TypeError("User defined agent is is not an instance of AgentRunner")


def main() -> int:
    """
    Load the agent from the provided path and run it against the benchmark

    Note that pytest creates a new global context when running tests.
    Because of this, we first load the agent and validate that it is of the correct type here.
    Then we pass the path to the agent runner and let it load it within the pytest context.
    Note your AgentRunner must be concurrency safe.
    """
    print_intro()

    # Load the agent
    args = parse_args()
    agent = load_agent_from_path(args.path)
    validate_agent_instance_available(agent)

    # Filter examples based on args
    filtered_examples = examples[:]
    if args.id:
        filtered_examples = [
            example for example in filtered_examples if example.id == args.id
        ]
    if args.intent:
        filtered_examples = [
            example for example in filtered_examples if example.type == args.intent
        ]
    if args.domain:
        filtered_examples = [
            example for example in filtered_examples if example.domain == args.domain
        ]

    # Test we actually have tests to run
    if len(filtered_examples) == 0:
        print()
        print("=======================================================================")
        print("🍌 No tests to run. Please ensure your filter parameters are correct 🍌")
        print("=======================================================================")
        return 0

    # Load the desired tests
    tests = [generate_test(example, args.headless) for example in filtered_examples]

    # Run the tests
    return run_tests(tests, args.path, args.s)


if __name__ == "__main__":
    main()



import json
from pathlib import Path
from typing import Any, Dict, List

from bananalyzer.data.schemas import Example

examples_path: Path = Path(__file__).resolve().parent / "examples.json"

with open(examples_path, "r", encoding="utf-8") as file:
    example_jsons: List[Dict[str, Any]] = json.load(file)

examples: List[Example] = [Example(**example) for example in example_jsons]


def get_all_example_urls() -> List[str]:
    return [example.url for example in examples]


def get_example_by_url(url: str) -> Example:
    matching_examples = [example for example in examples if example.url == url]
    if len(matching_examples) != 1:
        raise ValueError(
            f"Expected 1 matching example, got {len(matching_examples)}: {matching_examples}"
        )
    return matching_examples[0]


def get_examples_path() -> Path:
    return examples_path.parent
